﻿Ext.require('App.Const.Finances');
Ext.define('App.View.Kivii.Finances.Invoice.Debit.List', {
    extend: 'System.Control.EntityList',
    entityType: 'Kivii.Finances.Entities.Invoice',
    xtype: 'App.View.Kivii.Finances.Invoice.Debit.List',
    dataSource: '/Restful/Kivii.Finances.Entities.Invoice/QueryEx.json',
    requires: ['App.Control.DownloadField'],
    title: '借票管理',
    iconCls: 'x-fa fa-circle-o-notch',
    filter: {
        BeginTime: new Date(new Date().getFullYear(), new Date().getMonth(), 1),
        EndTime: new Date(new Date().getFullYear(), new Date().getMonth(), new Date().getDate() + 1),
        Debited: false,
        // Category: 'Debit'
        OwnerKvid: KiviiContext.CurrentMember.DepartmentKvid
    },
    //在末尾保留足够的滚动条大小的控件
    reserveScrollbar: true,
    //默认情况下启用缓冲渲染
    //启用缓存会造成空白行的问题,有些数据过多会预留渲染控件,切换成数据少的会预留空白
    bufferedRenderer: false,
    viewModel: {
        data: {
            _selection: null,
            paymentKvid: null,
            payment: {}
        }
    },
    bind: {
        selection: '{_selection}'
    },
    store: {
        autoLoad: true,
        pageSize: 400,
        sorters: [{
            property: 'OperateTime',
            direction: 'DESC'
        }]
    },
    selModel: {
        selType: 'checkboxmodel',
        checkboxSelect: true,
        pruneRemoved: false,
        extensible: 'y'
    },
    dockedItems: [{
        xtype: 'pagingtoolbar',
        dock: 'bottom',
        displayInfo: true,
        items: [{
            xtype: 'button',
            labelWidth: 60,
            iconCls: 'x-fa fa-check-square-o',
            bind: {
                text: '已选择{selectionCount}条记录,点击清除',
                hidden: '{selectionCount==0}'
            },
            handler: '_onClearSelections'
        }]
    }, {
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'scroller',
        items: [{
                xtype: 'combo',
                queryMode: 'local',
                displayField: 'displayInfo',
                valueField: 'debited',
                value: false,
                editable: false,
                filterPickList: true,
                reference: 'debited',
                store: {
                    fields: ['displayInfo', 'debited'],
                    data: [{
                        //     displayInfo: '全部',
                        //     debited: null
                        // }, {
                        displayInfo: '未到账借票',
                        debited: false
                    }, {
                        displayInfo: '已到账借票',
                        debited: true
                    }]
                },
                triggers: {
                    clear: {
                        weight: 2,
                        cls: 'x-fa fa-times',
                        handler: '_cleanDebited'
                    }
                },
                listeners: {
                    select: '_onSelectDebited'
                }
            }, {
                xtype: 'datefield',
                fieldLabel: '开票日期  开始',
                format: 'Y-m-d',
                labelWidth: 90,
                reference: 'startTime',
                editable: false,
                value: new Date(new Date().getFullYear(), new Date().getMonth(), 1),
                listeners: {
                    select: '_onSelectMonth'
                }
            },
            {
                xtype: 'datefield',
                fieldLabel: '结束',
                format: 'Y-m-d',
                labelWidth: 30,
                reference: 'endTime',
                editable: false,
                value: new Date(new Date().getFullYear(), new Date().getMonth(), new Date().getDate() + 1),
                listeners: {
                    select: '_onSelectMonth'
                }
            }, {
                xtype: 'searchField',
                flex: 1,
                minWidth: 250,
                labelWidth: 60,
                queryField: 'SerialNumber,PayerName,OperatorName',
                emptyText: '模糊搜索流水号、付款单位、申请人',
            }, {
                xtype: 'button',
                text: '指定搜索',
                iconCls: 'x-fa fa-search',
                handler: '_onSearch'
                // disabled:true
            }, {
                text: '新增借票',
                iconCls: 'x-fa fa-plus',
                iconAlign: 'left',
                handler: '_onAcceptInvoice'
            }, {
                xtype: 'splitbutton',
                text: '借票冲销',
                iconCls: 'x-fa fa-sign-in',
                handler: '_onDebiting',
                menu: [{
                    text: '冲销撤销',
                    iconCls: 'x-fa fa-sign-out',
                    handler: '_onDebitCancel'
                }],
                bind: {
                    disabled: '{!_selection}'
                }
            }, {
                text: '冲销记录',
                iconCls: 'x-fa fa-sign-in',
                handler: '_onDebitList',
                bind: {
                    disabled: '{!_selection}'
                }
            }, {
                xtype: 'downloadField',
                dataSource: '/Restful/Kivii.Finances.Entities.Invoice/QueryEx.xlsx', //数据源
                fileName: '数据导出',
                window: {
                    type: 'App.Control.Kivii.Finances.General.Template.Selector',
                    title: '选择模板',
                    iconCls: 'x-fa fa-file-excel-o',
                    autoScroll: true,
                    width: 700,
                    height: 350,
                    closeAction: 'hide',
                    modal: true,
                    layout: 'fit',
                    filter: {
                        FolderPath: '/Finances/Invoice/Files',
                        TagsContains: 'QueryEx'
                    }
                }
            }
        ]
    }],
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: true
        },
        items: [{
            xtype: 'rownumberer',
            align: 'center',
            resizable: false
        }, {
            text: '发票编码',
            dataIndex: 'SerialNumber',
            width: 90
        }, {
            text: '所属部门',
            dataIndex: 'OwnerName',
            width: 100
        }, {
            text: '发票抬头',
            dataIndex: 'PayerName',
            renderer: '_renderPayerName',
            width: 200
        }, {
            text: '金额(含税)',
            align: 'right',
            dataIndex: 'Amount',
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            width: 110
        }, {
            text: '税额',
            align: 'right',
            dataIndex: 'AmountTax',
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            width: 100
        }, {
            text: '金额(不含税)',
            align: 'right',
            dataIndex: 'AmountUntaxed',
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            width: 120
        }, {
            xtype: 'datecolumn',
            text: '开票日期',
            format: 'Y-m-d',
            dataIndex: 'OperateTime',
            width: 100
        }, {
            text: '收款金额',
            align: 'right',
            dataIndex: 'AmountPayment',
            renderer: '_renderAmount',
            width: 110
        }, {
            xtype: 'datecolumn',
            text: '收款日期',
            format: 'Y-m-d',
            dataIndex: 'PayedTime',
            width: 100
        }, {
            text: '申请人',
            dataIndex: 'OperatorName',
            // renderer: '_renderOperatorName',
            width: 140
        }, {
            text: '类型',
            dataIndex: 'Type',
            renderer: '_renderType',
            width: 130
        }, {
            text: '分类',
            dataIndex: 'Category',
            renderer: '_renderCategory',
            width: 90
        }, {
            text: '税号',
            dataIndex: 'PayerTaxNumber',
            width: 200
        }, {
            text: '地址',
            dataIndex: 'PayerCompanyAddress',
            renderer: '_renderPayerCompanyAddress',
            width: 260
        }, {
            text: '开户行',
            dataIndex: 'PayerRegisteredBank',
            renderer: '_renderPayerRegisteredBank',
            width: 260
        }, {
            text: '备注',
            dataIndex: 'Remark',
            minWidth: 180,
            flex: 1
        }]
    },
    listeners: {
        activate: '_onActivate'
    },
    controller: {
        _onClearSelections: function() {
            this.getView().clearSelections();
        },
        _onActivate: function(_ctrl, _eOpts) {
            this.getView().reload();
        },
        _renderAmount: function(_value, _meta, _record, _rowIndex) {
            if (_value === _record.data.Amount) {
                _meta.style = 'background-color:#B4EEB4';
            }
            return KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_value);
        },
        _renderOperatorName: function(_value, _meta, _record, _rowIndex) {
            if (_record.data.hasOwnProperty('OwnerName')) return _value + '(' + _record.data.OwnerName + ')';
            return _value;
        },
        _renderPayerName: function(_value, _meta, _record, _rowIndex) {
            if (_record.data.hasOwnProperty('OffsetKvid') && _record.data.OffsetKvid !== '00000000-0000-0000-0000-000000000000') {
                _meta.style = 'background-color:#FFC0CB';
                return _value + '[已作废]';
            }
            return _value;
        },
        _renderType: function(_value, _meta, _record, _rowIndex) {
            if (_value === "Debit") {
                _meta.style = 'background-color:#B4EEB4';
                return "存在对应收款";
            }
            return _value;
        },
        _renderCategory: function(_value, _meta, _record, _rowIndex) {
            if (_value === "Debit") {
                _meta.style = 'background-color:#F3F781';
                return "借票";
            }
            if (_value === "Payment") {
                _meta.style = 'background-color:#B4EEB4';
                return "到账";
            }
            return _value;
        },
        _renderPayerCompanyAddress: function(_value, _meta, _record, _rowIndex) {
            return _value + _record.data.PayerPhone;
        },
        _renderPayerRegisteredBank: function(_value, _meta, _record, _rowIndex) {
            return _value + _record.data.PayerBankAccount;
        },

        _onSelectDebited: function(_ctrl, _record) {
            let _controller = this;
            let _view = _controller.getView();
            delete _view.filter.Debited;
            _view.filter.Debited = _record.data.debited;
            _view.filter.OwnerKvid = KiviiContext.CurrentMember.DepartmentKvid;
            _view.reload();
        },
        _cleanDebited: function(_ctrl) {
            _ctrl.setValue('');
            let _controller = this;
            let _view = _controller.getView();
            if (_view.filter === null) _view.setFilter({});
            delete _view.filter.Debited;
            _view.filter.Debited = null;
            _view.filter.OwnerKvid = KiviiContext.CurrentMember.DepartmentKvid;
            _view.reload();
        },
        _onSelectMonth: function(_ctrl, _time, _eOpts) {
            let _controller = this;
            let _view = _controller.getView();
            let _refs = _controller.getReferences();
            let _startTime = _refs.startTime.value;
            let _endTime = _refs.endTime.value || new Date();
            if (_startTime === null) {
                Ext.toast("请选择开始时间!");
                return;
            }
            if (_startTime > _endTime) {
                Ext.toast("开始日期不能大于结束日期!");
                return;
            }
            delete _view.filter.BeginTime;
            delete _view.filter.EndTime;
            _view.filter.OwnerKvid = KiviiContext.CurrentMember.DepartmentKvid;
            _view.filter.BeginTime = new Date(_startTime.getFullYear(), _startTime.getMonth(), _startTime.getDate());
            _view.filter.EndTime = new Date(_endTime.getFullYear(), _endTime.getMonth(), _endTime.getDate() + 1);
            _view.reload();
        },


        _initEntity: function(_selecteds) {
            let _item = {};
            _item.Kvid=null;
            _item.OwnerKvid = KiviiContext.CurrentMember.DepartmentKvid;
            _item.OwnerName = KiviiContext.CurrentMember.DepartmentName;
            _item.OperateTime = new Date();
            _item.Type = "增值税专用发票";
            _item.TaxRate = 0.03;
            _item.Currency = "CNY";
            _item.PayerName = null;
            _item.PayeeName = KiviiContext.CurrentMember.OrganizationName;
            _item.PayeeTaxNumber = null;
            _item.Amount = 0;
            _item.Metadata = {};
            _item.Metadata.PayeeOperatorName = KiviiContext.CurrentMember.FullName;
            _item.Metadata.ReviewerName = KiviiContext.CurrentMember.FullName;
            _item.OperatorName = KiviiContext.CurrentMember.FullName;
            _item.Details = [];
            _item.Payments = [];
            _item.PaymentKvids = [];
            if (Ext.isArray(_selecteds)) {
                for (let _i = 0; _i < _selecteds.length; _i++) {
                    let _select = _selecteds[_i].data;
                    let _payment = {};
                    _payment.Kvid = _select.Kvid;
                    _payment.Amount = parseFloat(parseFloat(_select.Amount - _select.AmountInvoice).toFixed(2));
                    _item.Payments.push(_payment);
                    _item.PaymentKvids.push(_select.Kvid);
                    _item.Amount += _payment.Amount;
                    _item.PayerName = _select.PayerName;
                }
            }
            let _detail = {};
            _detail.GoodsId = "发票1";
            _detail.GoodsFullName = "检测费";
            _detail.GoodsModel = "3040601000000000000";
            _detail.TaxRate = _item.TaxRate;
            _detail.Quantity = 1;
            _detail.Amount = parseFloat(_item.Amount.toFixed(2));
            _detail.QuantityUnit = '次';
            _detail.AmountUntaxed = (parseFloat(_detail.Amount) / (1 + parseFloat(_detail.TaxRate))).toFixed(2);
            _detail.AmountTax = (parseFloat(_detail.Amount) - parseFloat(_detail.AmountUntaxed)).toFixed(2);
            _detail.QuantityUnitPrice = (parseFloat(_detail.Amount) / parseFloat(_detail.Quantity)).toFixed(2);
            _detail.QuantityUnitPriceUntaxed = (parseFloat(_detail.AmountUntaxed) / parseFloat(_detail.Quantity)).toFixed(2);
            _item.Details.push(_detail);
            return _item;
        },
        _onDebitList: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _selecteds = _vm.data.selections || [];
            if (!_view.selection) return;
            if (_selecteds.length <= 0) return;
            if (_selecteds.length > 1) {
                Ext.toast("查看记录时请勿多选!");
                return;
            }
            let _form = Ext.create('App.Control.Kivii.Finances.Invoice.Payed.List', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 1200,
                height: 650,
                modal: true,
                maximizable: true,
                resizable: true,
                items: [_form]
            });
            _form.setOwner(_view.selection.data, _ctrl);
            _win.show();
            _win.on('close', function(_ctrl, _entity) {
                _view.reload();
            });
        },
        _cleanSearch: function() {
            let _controller = this;
            let _view = _controller.getView();
            _view.filter.OwnerKvid = KiviiContext.CurrentMember.DepartmentKvid;
            _view.reload();
        },
        _onSearch: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            Ext.MessageBox.show({
                title: _ctrl.text,
                msg: '请批量传入报告号(支持逗号,回车,顿号,空格间隔):',
                width: 500,
                // height:300,
                buttons: Ext.MessageBox.OKCANCEL,
                multiline: true,
                scope: this,
                fn: this._showResultText,
                animateTarget: _ctrl
            });

        },
        _showResultText: function(_btn, _text, _ctrl) {
            if (_btn !== 'ok') return;
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _view = _controller.getView();
            let _store = _view.getStore();
            let _serialNumbers = _text.replace(/，/g, ',').replace(/、/g, ',').replace(/\n/g, ',');
            if (!_serialNumbers || _serialNumbers === "") return;

            Ext.Ajax.request({
                url: '/Restful/Kivii.Finances.Entities.Invoice/QueryEx.json',
                params: {
                    SerialNumberIn: _serialNumbers,
                    Debited: false
                },
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    _store.removeAll();
                    for (let _i = 0; _i < _rtns.Results.length; _i++) {
                        _store.add(_rtns.Results[_i]);
                    }
                },
                failure: function(_response) {}
            });
        },
        _onAcceptInvoice: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _item = _controller._initEntity();
            let _form = Ext.create('App.Control.Kivii.Finances.Invoice.Form', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 1200,
                height: 750,
                modal: true,
                maximizable: true,
                resizable: true,
                items: [_form]
            });
            _win.show();
            _form.setOwner(_item);
            _form.on('entityAccepted', function(_ctrl, _entity) {
                _win.close();
                _view.reload();
            });
        },
        _onDebiting: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _kvids = [];
            let _debitAmount = 0;
            let _select = _vm.data.selections || [];
            let _payerName = null;
            if (_select.length <= 0) {
                Ext.toast("请选中要冲销的借票!");
                return;
            }
            for (let _i = 0; _i < _select.length; _i++) {
                let _item = _select[_i].data;
                _kvids.push(_item.Kvid);
                _debitAmount += _item.Amount - _item.AmountPayment;
                _payerName = _item.PayerName;
            }
            if (_debitAmount <= 0) {
                Ext.toast("所选发票无需冲销!");
                return;
            }
            let _form = Ext.create('App.Control.Kivii.Finances.Debiting.Payment.Selector', {
                header: false
            });
            let _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: _ctrl.text,
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                // header:false,
                autoScroll: true,
                width: 900,
                height: 450,
                modal: true,
                resizable: false,
                items: [_form]
            });
            _win.show();
            _form.setOwner(_payerName);
            _form.on('entitySelected', function(_selector, _entity) {
                _win.close();
                let _payment = _entity[0];
                _vm.set('paymentKvid', _payment.RootKvid);
                _vm.set('payment', _payment);
                let _amount = _payment.Amount - _payment.AmountInvoice;
                let _msg = Ext.create('Ext.window.MessageBox', {
                    iconCls: _ctrl.iconCls,
                    iconAlign: 'left',
                    closeAction: 'destroy'
                });
                _msg.confirm(_ctrl.text, _payment.PayerName + "<br/>借票金额:<strong>" + _debitAmount.toFixed(2) + "</strong><br/>所选金额:<strong>" + _amount + "</strong>？", _controller._debitingInvoice, _controller);
            });
        },
        _debitingInvoice: function(_optional) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            if (_optional != "yes") {
                _vm.set('paymentKvid', null);
                return;
            }
            let _kvids = [];
            let _select = _vm.data.selections || [];
            if (_select.length <= 0) {
                Ext.toast("请选中要冲销的借票!");
                _vm.set('paymentKvid', null);
                return;
            }
            for (let _i = 0; _i < _select.length; _i++) {
                let _item = _select[_i].data;
                _kvids.push(_item.Kvid);
            }
            if (_vm.data.paymentKvid === null) return;
            let _jsonData = {};
            _jsonData.PaymentKvid = _vm.data.paymentKvid;
            _jsonData.InvoiceKvids = _kvids;
            Ext.Ajax.request({
                url: '/Restful/Kivii.Finances.Entities.Invoice/Debiting.json',
                jsonData: _jsonData,
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    _view.reload();
                    Ext.toast('冲销成功!');
                },
                failure: function(_response) {
                    _view.reload();
                }
            });
        },
        _onDebitCancel: function(_ctrl) {
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _kvids = [];
            let _select = _vm.data.selections || [];
            if (_select.length <= 0) {
                Ext.toast("请选中要撤销冲销的发票!");
                return;
            }
            let _msg = Ext.create('Ext.window.MessageBox', {
                iconCls: _ctrl.iconCls,
                iconAlign: 'left',
                closeAction: 'destroy'
            });
            _msg.confirm(_ctrl.text, "确认撤销冲销选中的发票?<br />撤销后发票将会以借票未到账的形式出现!", _controller._debitCancel, _controller);
        },
        _debitCancel: function(_optional) {
            if (_optional != "yes") {
                return;
            }
            let _controller = this;
            let _view = _controller.getView();
            let _vm = _controller.getViewModel();
            let _kvids = [];
            let _select = _vm.data.selections || [];
            if (_select.length <= 0) {
                Ext.toast("请选中要撤销冲销的发票!");
                return;
            }
            for (let _i = 0; _i < _select.length; _i++) {
                let _item = _select[_i].data;
                if (_item.hasOwnProperty('OffsetKvid') && _item.OffsetKvid !== '00000000-0000-0000-0000-000000000000') {
                    Ext.toast("所选发票已作废,无法撤销冲销!");
                    return;
                }
                _kvids.push(_item.Kvid);
            }
            let _jsonData = {};
            _jsonData.InvoiceKvids = _kvids;
            Ext.Ajax.request({
                url: '/Restful/Kivii.Finances.Entities.Invoice/DebitCancel.json',
                jsonData: _jsonData,
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    _view.reload();
                    Ext.toast('撤销成功!');
                },
                failure: function(_response) {
                    _view.reload();
                }
            });
        }
    }
});